@@ -77,6 +77,7 @@ schro_engine_code_picture (SchroEncoderFrame *frame,
       if (encoder->reference_pictures[i] == NULL) break;
       if (encoder->reference_pictures[i]->frame_number == retire) {
         schro_encoder_frame_unref(encoder->reference_pictures[i]);
+        encoder->reference_pictures[i] = NULL;
         break;
       }
     }
@@ -367,7 +368,8 @@ init_params (SchroEncoderFrame *frame)
 
   schro_params_init (params, params->video_format->index);
 
-  if (encoder->enable_noarith && frame->num_refs == 0) {
+  if ((encoder->enable_noarith && frame->num_refs == 0) ||
+      params->is_lowdelay) {
     params->is_noarith = TRUE;
   }
 
