@@ -9,6 +9,7 @@
 
 int _schro_motion_ref = FALSE;
 
+#ifdef ENABLE_MOTION_REF
 static int
 get_pixel (SchroMotion *motion, int k, SchroUpsampledFrame *upframe,
     int x, int y, int dx, int dy);
@@ -16,6 +17,7 @@ get_pixel (SchroMotion *motion, int k, SchroUpsampledFrame *upframe,
 int
 schro_motion_pixel_predict_block (SchroMotion *motion, int x, int y, int k,
     int i, int j);
+#endif
 
 
 SchroMotion *
@@ -46,6 +48,7 @@ schro_motion_free (SchroMotion *motion)
   schro_free (motion);
 }
 
+#ifdef ENABLE_MOTION_REF
 static void
 schro_motion_get_global_vector (SchroMotion *motion, int ref, int x, int y,
     int *dx, int *dy)
@@ -168,7 +171,6 @@ get_biref_pixel (SchroMotion *motion, int i, int j, int k, int x, int y)
   return ROUND_SHIFT(value, motion->ref_weight_precision);
 }
 
-
 static int
 get_pixel (SchroMotion *motion, int k, SchroUpsampledFrame *upframe,
     int x, int y, int dx, int dy)
@@ -186,6 +188,7 @@ get_pixel (SchroMotion *motion, int k, SchroUpsampledFrame *upframe,
   return schro_upsampled_frame_get_pixel_precN (upframe, k, px, py,
       motion->mv_precision);
 }
+#endif
 
 static int
 get_ramp (int x, int offset)
@@ -197,7 +200,7 @@ get_ramp (int x, int offset)
   return 1 + (6 * x + offset - 1)/(2*offset - 1);
 }
 
-
+#ifdef ENABLE_MOTION_REF
 int
 schro_motion_pixel_predict_block (SchroMotion *motion, int x, int y, int k,
     int i, int j)
@@ -268,6 +271,7 @@ schro_motion_pixel_predict_block (SchroMotion *motion, int x, int y, int k,
 
   return value * wx * wy;
 }
+#endif
 
 /* motion render (faster) */
 
@@ -277,6 +281,7 @@ get_block (SchroMotion *motion, int k, int ref, int i, int j, int dx, int dy)
   int px, py;
   int x, y;
   SchroUpsampledFrame *upframe;
+  int exp;
 
   if (k > 0) {
     dx >>= SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
@@ -292,15 +297,14 @@ get_block (SchroMotion *motion, int k, int ref, int i, int j, int dx, int dy)
   y = motion->ybsep * j - motion->yoffset;
   px = (x << motion->mv_precision) + dx;
   py = (y << motion->mv_precision) + dy;
+  exp = 32 << motion->mv_precision;
 
-  if (px >= 0 && py >= 0 &&
-      px < motion->max_fast_x && py < motion->max_fast_y) {
-    schro_upsampled_frame_get_block_fast_precN (upframe, k, px, py,
-        motion->mv_precision, &motion->tmp_block_ref[ref]);
-  } else {
-    schro_upsampled_frame_get_block_precN (upframe, k, px, py,
-        motion->mv_precision, &motion->tmp_block_ref[ref]);
-  }
+  px = CLAMP (px, -exp, motion->max_fast_x + exp-1);
+  py = CLAMP (py, -exp, motion->max_fast_y + exp-1);
+
+  schro_upsampled_frame_get_block_fast_precN (upframe, k, px, py,
+      motion->mv_precision, &motion->block_ref[ref],
+      &motion->alloc_block_ref[ref]);
 }
 
 static void
@@ -314,9 +318,11 @@ get_dc_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   mvdc = (SchroMotionVectorDC *)
     &motion->motion_vectors[j*params->x_num_blocks + i];
 
+  memcpy (&motion->block, &motion->alloc_block, sizeof(SchroFrameData));
   value = mvdc->dc[k];
   for(jj=0;jj<motion->yblen;jj++) {
     uint8_t *data = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
+    /* FIXME splat */
     for(ii=0;ii<motion->xblen;ii++) {
       data[ii] = value + 128;
     }
@@ -341,15 +347,14 @@ get_ref1_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   shift = motion->ref_weight_precision;
 
   if (weight == (1<<shift)) {
-    for(jj=0;jj<motion->yblen;jj++) {
-      uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-      uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[0], jj);
-      memcpy(d,s,motion->xblen);
-    }
+    memcpy (&motion->block, &motion->block_ref[0],
+        sizeof(SchroFrameData));
   } else {
+    memcpy (&motion->block, &motion->alloc_block,
+        sizeof(SchroFrameData));
     for(jj=0;jj<motion->yblen;jj++) {
       uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-      uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[0], jj);
+      uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[0], jj);
       for(ii=0;ii<motion->xblen;ii++) {
         d[ii] = ROUND_SHIFT(s[ii] * weight, shift);
       }
@@ -375,15 +380,14 @@ get_ref2_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   shift = motion->ref_weight_precision;
 
   if (weight == (1<<shift)) {
-    for(jj=0;jj<motion->yblen;jj++) {
-      uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-      uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[1], jj);
-      memcpy(d,s,motion->xblen);
-    }
+    memcpy (&motion->block, &motion->block_ref[1],
+        sizeof(SchroFrameData));
   } else {
+    memcpy (&motion->block, &motion->alloc_block,
+        sizeof(SchroFrameData));
     for(jj=0;jj<motion->yblen;jj++) {
       uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-      uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[1], jj);
+      uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[1], jj);
       for(ii=0;ii<motion->xblen;ii++) {
         d[ii] = ROUND_SHIFT(s[ii] * weight, shift);
       }
@@ -410,31 +414,38 @@ get_biref_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   weight1 = motion->ref2_weight;
   shift = motion->ref_weight_precision;
 
+  memcpy (&motion->block, &motion->alloc_block, sizeof(SchroFrameData));
   if (weight0 == 1 && weight1 == 1 && shift == 1) {
     switch (motion->xblen) {
       case 8:
         oil_avg2_8xn_u8(motion->block.data, motion->block.stride,
-            motion->tmp_block_ref[0].data, motion->tmp_block_ref[0].stride,
-            motion->tmp_block_ref[1].data, motion->tmp_block_ref[1].stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       case 12:
         oil_avg2_12xn_u8(motion->block.data, motion->block.stride,
-            motion->tmp_block_ref[0].data, motion->tmp_block_ref[0].stride,
-            motion->tmp_block_ref[1].data, motion->tmp_block_ref[1].stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       case 16:
         oil_avg2_16xn_u8(motion->block.data, motion->block.stride,
-            motion->tmp_block_ref[0].data, motion->tmp_block_ref[0].stride,
-            motion->tmp_block_ref[1].data, motion->tmp_block_ref[1].stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            motion->yblen);
+        break;
+      case 32:
+        oil_avg2_32xn_u8(motion->block.data, motion->block.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       default:
         for(jj=0;jj<motion->yblen;jj++) {
           uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-          uint8_t *s0 = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[0], jj);
-          uint8_t *s1 = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[1], jj);
+          uint8_t *s0 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[0], jj);
+          uint8_t *s1 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[1], jj);
           for(ii=0;ii<motion->xblen;ii++) {
             d[ii] = (s0[ii] + s1[ii] + 1)>>1;
           }
@@ -451,27 +462,27 @@ get_biref_block (SchroMotion *motion, int i, int j, int k, int x, int y)
     switch (motion->xblen) {
       case 8:
         oil_combine2_8xn_u8(motion->block.data, motion->block.stride,
-            motion->tmp_block_ref[0].data, motion->tmp_block_ref[0].stride,
-            motion->tmp_block_ref[1].data, motion->tmp_block_ref[1].stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
             as, motion->yblen);
         break;
       case 12:
         oil_combine2_16xn_u8(motion->block.data, motion->block.stride,
-            motion->tmp_block_ref[0].data, motion->tmp_block_ref[0].stride,
-            motion->tmp_block_ref[1].data, motion->tmp_block_ref[1].stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
             as, motion->yblen);
         break;
       case 16:
         oil_combine2_16xn_u8(motion->block.data, motion->block.stride,
-            motion->tmp_block_ref[0].data, motion->tmp_block_ref[0].stride,
-            motion->tmp_block_ref[1].data, motion->tmp_block_ref[1].stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
             as, motion->yblen);
         break;
       default:
         for(jj=0;jj<motion->yblen;jj++) {
           uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-          uint8_t *s0 = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[0], jj);
-          uint8_t *s1 = SCHRO_FRAME_DATA_GET_LINE (&motion->tmp_block_ref[1], jj);
+          uint8_t *s0 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[0], jj);
+          uint8_t *s1 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[1], jj);
           for(ii=0;ii<motion->xblen;ii++) {
             d[ii] = ROUND_SHIFT(s0[ii] * weight0 + s1[ii] * weight1, shift);
           }
@@ -574,7 +585,7 @@ schro_motion_block_accumulate_slow (SchroMotion *motion, SchroFrameData *comp,
   for(j=0;j<motion->yblen;j++) {
     int16_t *d = SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y + j);
     uint8_t *s = SCHRO_FRAME_DATA_GET_LINE (&motion->block, j);
-    
+
     if (y + j < 0 || y + j >= comp->height) continue;
 
     w_y = motion->weight_y[j];
@@ -605,7 +616,7 @@ schro_motion_block_accumulate_slow (SchroMotion *motion, SchroFrameData *comp,
 }
 
 void
-init_obmc_weight (SchroMotion *motion)
+schro_motion_init_obmc_weight (SchroMotion *motion)
 {
   int i;
   int j;
@@ -656,11 +667,51 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
   SchroParams *params = motion->params;
   int max_x_blocks;
   int max_y_blocks;
+  int16_t zero = 0;
 
+#ifdef ENABLE_MOTION_REF
   if (_schro_motion_ref) {
     schro_motion_render_ref (motion, dest);
     return;
   }
+#endif
+
+  if (params->have_global_motion) {
+#ifdef ENABLE_MOTION_REF
+    SCHRO_WARNING ("global motion enabled, using reference motion renderer");
+    schro_motion_render_ref (motion, dest);
+    return;
+#else
+    SCHRO_ERROR ("global motion enabled, probably will crash");
+#endif
+  }
+
+  {
+    int min_extension;
+    int i;
+
+    min_extension = motion->src1->frames[0]->extension;
+    for(i=0;i<4;i++){
+      if (motion->src1->frames[i]) {
+        min_extension = MIN(min_extension, motion->src1->frames[i]->extension);
+      }
+      if (motion->src2 && motion->src2->frames[i]) {
+        min_extension = MIN(min_extension, motion->src2->frames[i]->extension);
+      }
+    }
+
+    if (MAX(params->xblen_luma, params->yblen_luma) > min_extension) {
+#ifdef ENABLE_MOTION_REF
+      SCHRO_WARNING ("block size (%dx%d) larger than minimum frame extension %d, using reference motion renderer",
+          params->xblen_luma, params->yblen_luma, min_extension);
+      schro_motion_render_ref (motion, dest);
+      return;
+#else
+      SCHRO_ERROR ("block size (%dx%d) larger than minimum frame extension %d, probably will crash",
+          params->xblen_luma, params->yblen_luma, min_extension);
+#endif
+    }
+  }
 
   if (params->num_refs == 1) {
     SCHRO_ASSERT(params->picture_weight_2 == 1);
@@ -699,27 +750,28 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
     motion->max_fast_x = (motion->width - motion->xblen) << motion->mv_precision;
     motion->max_fast_y = (motion->height - motion->yblen) << motion->mv_precision;
 
-    motion->block.data = schro_malloc (motion->xblen * motion->yblen * sizeof(uint8_t));
-    motion->block.stride = motion->xblen * sizeof(uint8_t);
-    motion->block.width = motion->xblen;
-    motion->block.height = motion->yblen;
+    motion->alloc_block.data = schro_malloc (motion->xblen * motion->yblen * sizeof(uint8_t));
+    motion->alloc_block.stride = motion->xblen * sizeof(uint8_t);
+    motion->alloc_block.width = motion->xblen;
+    motion->alloc_block.height = motion->yblen;
     motion->obmc_weight.data = schro_malloc (motion->xblen * motion->yblen * sizeof(int16_t));
     motion->obmc_weight.stride = motion->xblen * sizeof(int16_t);
     motion->obmc_weight.width = motion->xblen;
     motion->obmc_weight.height = motion->yblen;
-    motion->tmp_block_ref[0].data = schro_malloc (motion->xblen * motion->yblen * sizeof(uint8_t));
-    motion->tmp_block_ref[0].stride = motion->xblen * sizeof(uint8_t);
-    motion->tmp_block_ref[0].width = motion->xblen;
-    motion->tmp_block_ref[0].height = motion->yblen;
-    motion->tmp_block_ref[1].data = schro_malloc (motion->xblen * motion->yblen * sizeof(uint8_t));
-    motion->tmp_block_ref[1].stride = motion->xblen * sizeof(uint8_t);
-    motion->tmp_block_ref[1].width = motion->xblen;
-    motion->tmp_block_ref[1].height = motion->yblen;
+    motion->alloc_block_ref[0].data = schro_malloc (motion->xblen * motion->yblen * sizeof(uint8_t));
+    motion->alloc_block_ref[0].stride = motion->xblen * sizeof(uint8_t);
+    motion->alloc_block_ref[0].width = motion->xblen;
+    motion->alloc_block_ref[0].height = motion->yblen;
+    motion->alloc_block_ref[1].data = schro_malloc (motion->xblen * motion->yblen * sizeof(uint8_t));
+    motion->alloc_block_ref[1].stride = motion->xblen * sizeof(uint8_t);
+    motion->alloc_block_ref[1].width = motion->xblen;
+    motion->alloc_block_ref[1].height = motion->yblen;
 
-    init_obmc_weight(motion);
+    schro_motion_init_obmc_weight (motion);
 
     for(j=0;j<comp->height;j++){
-      memset (SCHRO_FRAME_DATA_GET_LINE(comp, j), 0, comp->width * sizeof(int16_t));
+      oil_splat_s16_ns (SCHRO_FRAME_DATA_GET_LINE(comp, j), &zero,
+          comp->width);
     }
 
     max_x_blocks = MIN(params->x_num_blocks - 1,
@@ -781,10 +833,10 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
           SCHRO_FRAME_DATA_GET_LINE(comp, j), as, motion->width);
     }
 
-    schro_free (motion->block.data);
+    schro_free (motion->alloc_block.data);
     schro_free (motion->obmc_weight.data);
-    schro_free (motion->tmp_block_ref[0].data);
-    schro_free (motion->tmp_block_ref[1].data);
+    schro_free (motion->alloc_block_ref[0].data);
+    schro_free (motion->alloc_block_ref[1].data);
   }
 
 }
@@ -1030,10 +1082,6 @@ schro_motion_verify (SchroMotion *motion)
       mv = &motion->motion_vectors[y*params->x_num_blocks + x];
       sbmv = &motion->motion_vectors[(y&~3)*params->x_num_blocks + (x&~3)];
 
-      SCHRO_DEBUG("verify %d %d: %d %d  %d %d %d %d  %d %d %d %d", x, y, mv->split, mv->pred_mode,
-          mv->using_global, mv->unused, mv->scan, mv->metric,
-          mv->dx[0], mv->dy[0], mv->dx[1], mv->dy[1]);
-
       if (mv->split != sbmv->split) {
         SCHRO_ERROR("mv(%d,%d) has the wrong split", x, y);
         return 0;
@@ -1099,6 +1147,7 @@ schro_motion_verify (SchroMotion *motion)
   return 1;
 }
 
+#ifdef ENABLE_MOTION_REF
 void
 schro_motion_render_ref (SchroMotion *motion, SchroFrame *dest)
 {
@@ -1150,4 +1199,5 @@ schro_motion_render_ref (SchroMotion *motion, SchroFrame *dest)
     }
   }
 }
+#endif
 
